@@ -2,25 +2,27 @@
 from __future__ import unicode_literals
 
 MESSAGES = {
-    "Also available in": "Također dostupno i u",
+    "%d min remaining to read": "",
     "Also available in:": "Također dostupno i u:",
     "Archive": "Arhiva",
     "Categories": "Kategorije",
+    "Comments": "Komentari",
     "LANGUAGE": "hrvatski",
+    "Languages:": "Jezici:",
     "More posts about %s": "Više postova o %s",
-    "More posts about": "Više postova o",
     "Newer posts": "Noviji postovi",
     "Next post": "Sljedeći post",
-    "No posts found.": "",
-    "Nothing found.": "",
+    "No posts found.": "Nema postova.",
+    "Nothing found.": "Nema ničeg.",
     "Older posts": "Stariji postovi",
     "Original site": "Izvorna stranica",
-    "Posted": "Objavljeno",
     "Posted:": "Objavljeno:",
     "Posts about %s": "Postovi o %s",
     "Posts for year %s": "Postovi za godinu %s",
     "Posts for {month} {year}": "Postovi za {month} {year}",
     "Previous post": "Prethodni post",
+    "Publication date": "Nadnevak objave",
+    "RSS feed": "RSS kanal",
     "Read in English": "Čitaj na hrvatskom",
     "Read more": "Čitaj dalje",
     "Source": "Izvor",
