@@ -2,25 +2,27 @@
 from __future__ import unicode_literals
 
 MESSAGES = {
-    "Also available in": "Также доступно на",
+    "%d min remaining to read": "",
     "Also available in:": "Также доступно на:",
     "Archive": "Архив",
     "Categories": "Категории",
+    "Comments": "",
     "LANGUAGE": "Русский",
+    "Languages:": "",
     "More posts about %s": "Больше записей о %s",
-    "More posts about": "Больше записей о",
     "Newer posts": "Новые записи",
     "Next post": "Следующая запись",
     "No posts found.": "",
     "Nothing found.": "",
     "Older posts": "Старые записи",
     "Original site": "Оригинальный сайт",
-    "Posted": "Опубликовано",
     "Posted:": "Опубликовано:",
     "Posts about %s": "Записи о %s",
     "Posts for year %s": "Записи за %s год",
     "Posts for {month} {year}": "Записи за {month} {year}",
     "Previous post": "Предыдущая запись",
+    "Publication date": "",
+    "RSS feed": "",
     "Read in English": "Прочесть по-русски",
     "Read more": "Читать далее",
     "Source": "Источник",
