@@ -2,25 +2,27 @@
 from __future__ import unicode_literals
 
 MESSAGES = {
-    "Also available in": "ان زبانوں میں بھی دستیاب",
+    "%d min remaining to read": "",
     "Also available in:": "ان زبانوں میں بھی دستیاب:",
     "Archive": "آرکائیو",
     "Categories": "زمرے",
+    "Comments": "",
     "LANGUAGE": "اردو",
+    "Languages:": "",
     "More posts about %s": "%s کے بارے میں مزید تحاریر",
-    "More posts about": " کے بارے میں مزید تحاریر",
     "Newer posts": "نئی تحاریر",
     "Next post": "اگلی تحریر",
     "No posts found.": "",
     "Nothing found.": "",
     "Older posts": "پرانی تحاریر",
     "Original site": "اصلی سائٹ",
-    "Posted": "اشاعت",
     "Posted:": "اشاعت:",
     "Posts about %s": "%s کے بارے میں تحاریر",
     "Posts for year %s": "سال %s کی تحاریر",
     "Posts for {month} {year}": "{month} {year} کی تحاریر",
     "Previous post": "پچھلی تحریر",
+    "Publication date": "",
+    "RSS feed": "",
     "Read in English": "اردو میں پڑھیے",
     "Read more": "مزید پڑھیے",
     "Source": "سورس",
